package com.example.afrishop_v3.controllers;

import com.braintreegateway.*;

import com.example.afrishop_v3.models.TbCfOrder;
import com.example.afrishop_v3.repository.TbCfOrderRepository;
import com.example.afrishop_v3.util.GatewayFactory;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Map;
import java.util.Optional;

/**
 * @Auther: wudepeng
 * @Date: 2021/02/27
 * @Description:Braintree支付
 */
@RestController
@RequestMapping("/braintree")
public class BraintreeController {

    private static BraintreeGateway gateway = GatewayFactory.getlisiGateway();
    private final TbCfOrderRepository orderRepository;
    @Value("${paypal.success_page}")
    private String PAYPAL_SUCCESS_PAGE;

    @Value("${paypal.failed_page}")
    private String PAYPAL_FAILED_PAGE;

    public BraintreeController(TbCfOrderRepository orderRepository) {
        this.orderRepository = orderRepository;
    }

    @PostMapping("/getToken")
    public String getToken() {
        //官方写new ClientTokenRequest().customerId("customerId")可以不要。
        ClientTokenRequest clientTokenRequest = new ClientTokenRequest();
        String clientToken = gateway.clientToken().generate(clientTokenRequest);
        return clientToken;
    }

    @PostMapping("/pay")
    public void pay(@RequestParam String orderId,
                    @RequestParam String nonce,
                    HttpServletRequest req,
                    HttpServletResponse resp) throws IOException {

        Optional<TbCfOrder> byId = orderRepository.findById(orderId);

        if (StringUtils.isBlank(orderId) || StringUtils.isBlank(nonce) || !byId.isPresent()) {
            resp.sendRedirect(PAYPAL_FAILED_PAGE);
        }
        TbCfOrder order = byId.get();
        BigDecimal orderPrice = order.getRealityPay();
        String orderInfo = order.getDeliveryName() + "'s payment information";
        TransactionRequest request = new TransactionRequest()
                .amount(orderPrice)
                .paymentMethodNonce(nonce)
                .deviceData(orderInfo)
                .options()
                .submitForSettlement(true)
                .done();

        Result<Transaction> result = gateway.transaction().sale(request);
        System.err.println(JSONObject.fromObject(result));
        System.out.println("=============================================");
        System.out.println("支付状态：" + result.isSuccess());

        if (result.isSuccess()) {
            resp.sendRedirect(PAYPAL_SUCCESS_PAGE);
        } else {
            resp.sendRedirect(PAYPAL_FAILED_PAGE);
        }


    }


}
